#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
{short description of this unit test}
"""

from __future__ import absolute_import

import re
import unittest
from mock import Mock

from zhmcclient import ...

class Test...(unittest.TestCase):

    def setUp(self):
        self..... # set up test object

    def test_{case1}(self):
        ... # execute the code to be tested
        ... # assert expected results using assert statement

    ... # more test functions

if __name__ == '__main__':
    unittest.main()
